==> 0.0.2.tar.gz: lib/src/default_editors.dart <==

library fields.default_editors;

import 'package:tags/html.dart';
import 'package:fields/field.dart';
import 'package:fields/field_editor.dart';

bool registerDefaultEditors() {  
  if (!FieldEditor.hasDefault)
    FieldEditor.registerDefault(DefaultEditor);  
  if (!FieldEditor.has(bool)) 
    FieldEditor.register(bool, BoolEditor);
  if (!FieldEditor.has(DateTime))
    FieldEditor.register(DateTime, DateTimeEditor);
  if (!FieldEditor.has(num))
    FieldEditor.register(num, NumberEditor);
}

class UseRange extends Annotation {
  final num min, max, step;
  const UseRange({this.min, this.max, this.step});
}

class BoolEditor extends FieldEditor {  
  final Type valueType = bool;  
  final CheckboxInputElement _input = input()..type = 'checkbox';
  Element get element => _input;
  
  BoolEditor(Field f) : super(f) {
    _input.disabled = isValueReadOnly;
    _input.onChange.listen((_) => setValue(_input.checked));
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.checked = v);
}

class ColorEditorResources extends Resources {
  String get style => super.style + 
"""
.picker {  
  float:left;
}
.valuebox { 
  overflow:hidden; 
}
""";
  const ColorEditorResources();
}

class ColorEditor extends FieldEditor {
  final Type valueType = String;
  InputElement _picker;
  InputElement _display;
  Element _element;
  Element get element => _element;
  
  ColorEditor(Field f) : super(f, const ColorEditorResources()) {
    _element = span(() {
      style()
        ..scoped = true
        ..text = resources.style;   
      _picker = input()
          ..type = 'color'
          ..classes = ['picker']
          ..disabled = isValueReadOnly
          ..onChange.listen((_) {
            _display.value = _picker.value;
            setValue(_picker.value);            
          });
      div(() {
        _display = input()..type = 'text'..disabled = true;
      })..classes=['valuebox'];
    });
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) {
    _picker.value = v;
    _display.value = v;
  });
}

class DateTimeEditor extends FieldEditor {
  final Type valueType = DateTime;
  DateInputElement _input;
  Element get element => _input;
  
  DateTimeEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'date' // TODO(rms): investigate 'datetime-local'
      ..disabled = isValueReadOnly
      ..onChange.listen((_) => setValue(_input.valueAsDate));
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.valueAsDate = v);
}

class NumberEditor extends FieldEditor {
  final Type valueType = num;
  InputElement _input;
  Element get element => _input;
  
  NumberEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'number'
      ..disabled = isValueReadOnly
      ..onChange.listen((_) => setValue(_input.valueAsNumber));
    
    final match = (a) => a is UseRange;
    if (f.annotations.any(match)) {
      UseRange range = f.annotations.firstWhere(match);
      if (range.min != null) _input.min = range.min.toString();      
      if (range.max != null) _input.max = range.max.toString();      
      if (range.step != null) _input.step = range.step.toString();
    }
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.valueAsNumber = v);
}

class RangeEditorResources extends Resources {
  String get style => super.style + 
"""
.range {  
  float:left;
}
.valuebox { 
  overflow:hidden; 
}
""";
  const RangeEditorResources();
}

class RangeEditor extends FieldEditor {
  final Type valueType = num;
  InputElement _range;
  InputElement _display;
  Element _element;
  Element get element => _element;
  
  RangeEditor(Field f) : super(f, const RangeEditorResources()) {
    _element = span(() {
      style()
        ..scoped = true
        ..text = resources.style;   
      
      _range = input()
          ..type = 'range'
          ..classes = ['range']
          ..disabled = isValueReadOnly
          ..onChange.listen((_) {
            _display.value = _range.value;
            setValue(_range.valueAsNumber);            
          });
      
      div(() {
        _display = input()..type = 'text'..disabled = true;
      })..classes=['valuebox'];
    });

    final match = (a) => a is UseRange;
    if (f.annotations.any(match)) {
      UseRange range = f.annotations.firstWhere(match);
      if (range.min != null) _range.min = range.min.toString();      
      if (range.max != null) _range.max = range.max.toString();      
      if (range.step != null) _range.step = range.step.toString();
    }
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) {
    _range.valueAsNumber = v;
    _display.value = v.toString();
  });
}

class DefaultEditor extends FieldEditor {   
  final Type valueType = String; 
  InputElement _input;
  Element get element => _input;
    
  DefaultEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'text'
      ..disabled = isValueReadOnly
      ..onInput.listen((_) => 
        setValue(_input.value)
          .then((_) => _input.classes.remove('error'))
          .catchError((_) => _input.classes.add('error')));
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.value = v);
}

==> 0.0.4.tar.gz: lib/src/default_editors.dart <==

library fields.default_editors;

import 'dart:html';
import 'package:tags/html.dart';
import 'package:fields/field.dart';
import 'package:fields/field_editor.dart';

void registerDefaultEditors() {  
  if (!FieldEditor.isDefaultRegistered)
    FieldEditor.registerDefault(DefaultEditor);  
  if (!FieldEditor.isRegistered(bool)) 
    FieldEditor.register(bool, BoolEditor);
  if (!FieldEditor.isRegistered(DateTime))
    FieldEditor.register(DateTime, DateTimeEditor);
  if (!FieldEditor.isRegistered(num))
    FieldEditor.register(num, NumberEditor);
}

class UseRange extends Annotation {
  final num min, max, step;
  const UseRange({this.min, this.max, this.step});
}

class BoolEditor extends FieldEditor {  
  final Type valueType = bool;  
  final CheckboxInputElement _input = input()..type = 'checkbox';
  Element get element => _input;
  
  BoolEditor(Field f) : super(f) {
    _input.disabled = isValueReadOnly;
    _input.onChange.listen((_) => setValue(_input.checked));
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.checked = v);
}

class ColorEditorResources extends FieldEditorResources {
  String get style => super.style + 
"""
.picker {  
  float:left;
}
.valuebox { 
  overflow:hidden; 
}
""";
  const ColorEditorResources();
}

class ColorEditor extends FieldEditor {
  final Type valueType = String;
  InputElement _picker;
  InputElement _display;
  Element _element;
  Element get element => _element;
  
  ColorEditor(Field f) : super(f, const ColorEditorResources()) {
    _element = span(() {
      style()
        ..scoped = true
        ..text = resources.style;   
      _picker = input()
          ..type = 'color'
          ..classes = ['picker']
          ..disabled = isValueReadOnly
          ..onChange.listen((_) {
            _display.value = _picker.value;
            setValue(_picker.value);            
          });
      div(() {
        _display = input()..type = 'text'..disabled = true;
      })..classes=['valuebox'];
    });
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) {
    _picker.value = v;
    _display.value = v;
  });
}

class DateTimeEditor extends FieldEditor {
  final Type valueType = DateTime;
  DateInputElement _input;
  Element get element => _input;
  
  DateTimeEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'date' // TODO(rms): investigate 'datetime-local'
      ..disabled = isValueReadOnly
      ..onChange.listen((_) => setValue(_input.valueAsDate));
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.valueAsDate = v);
}

class NumberEditor extends FieldEditor {
  final Type valueType = num;
  InputElement _input;
  Element get element => _input;
  
  NumberEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'number'
      ..disabled = isValueReadOnly
      ..onChange.listen((_) => setValue(_input.valueAsNumber));
    
    final match = (a) => a is UseRange;
    if (f.annotations.any(match)) {
      UseRange range = f.annotations.firstWhere(match);
      if (range.min != null) _input.min = range.min.toString();      
      if (range.max != null) _input.max = range.max.toString();      
      if (range.step != null) _input.step = range.step.toString();
    }
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.valueAsNumber = v);
}

class RangeEditorResources extends FieldEditorResources {
  String get style => super.style + 
"""
.range {  
  float:left;
}
.valuebox { 
  overflow:hidden; 
}
""";
  const RangeEditorResources();
}

class RangeEditor extends FieldEditor {
  final Type valueType = num;
  InputElement _range;
  InputElement _display;
  Element _element;
  Element get element => _element;
  
  RangeEditor(Field f) : super(f, const RangeEditorResources()) {
    _element = span(() {
      style()
        ..scoped = true
        ..text = resources.style;   
      
      _range = input()
          ..type = 'range'
          ..classes = ['range']
          ..disabled = isValueReadOnly
          ..onChange.listen((_) {
            _display.value = _range.value;
            setValue(_range.valueAsNumber);            
          });
      
      div(() {
        _display = input()..type = 'text'..disabled = true;
      })..classes=['valuebox'];
    });

    final match = (a) => a is UseRange;
    if (f.annotations.any(match)) {
      UseRange range = f.annotations.firstWhere(match);
      if (range.min != null) _range.min = range.min.toString();      
      if (range.max != null) _range.max = range.max.toString();      
      if (range.step != null) _range.step = range.step.toString();
    }
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) {
    _range.valueAsNumber = v;
    _display.value = v.toString();
  });
}

class DefaultEditor extends FieldEditor {   
  final Type valueType = String; 
  InputElement _input;
  Element get element => _input;
    
  DefaultEditor(Field f) : super(f) {
    _input = input(() {
      style()
        ..scoped = true
        ..text = resources.style; 
    })..type = 'text'
      ..disabled = isValueReadOnly
      ..onInput.listen((_) => 
        setValue(_input.value)
          .then((_) => _input.classes.remove('error'))
          .catchError((_) => _input.classes.add('error')));
    
    f.values.listen((_) => _render());
    _render();
  }
  
  _render() => getValue().then((v) => _input.value = v);
}
